import tkinter as tk
from tkinter import filedialog, ttk, simpledialog
import os
import pandas as pd
from docx import Document
from docx.shared import RGBColor
from docx.shared import Pt
from docx.enum.text import WD_PARAGRAPH_ALIGNMENT
import re

# 浏览文件函数(0)
def browse_file(entry):
    file_path = filedialog.askopenfilename()
    entry.delete(0, tk.END)
    entry.insert(0, file_path)

# 浏览文件夹函数(1)
def browse_folder(entry):
    folder_path = filedialog.askdirectory()
    entry.delete(0, tk.END)
    entry.insert(0, folder_path)

# 简并碱基处理函数(2-1)
def start_process_DeleteDBs():
    input_file = sequences_entry_DeleteDBs.get()
    output_file = output_entry_DeleteDBs.get()
    # 调用简并碱基处理的函数
    process_fasta_file(input_file, output_file)
    success_label_DeleteDBs.config(text="Degenerate bases processed successfully!")

# Specificity&Recall rate计算预处理函数(2-2)
def start_process_SandPpreprocessing():
    input_file_path = sequences_entry_SandPpreprocessing.get()
    output_folder_path = output_entry_SandPpreprocessing.get()
    # 调用Specificity&Recall rate计算预处理的函数
    output_files = process_and_export_sequences(input_file_path, output_folder_path)
    success_label_SandPpreprocessing.config(text="Specificity and recall rate preprocessing completed!")

# 召回率处理函数(2-3)
def start_process_recall():
    barcode_path = barcode_entry.get()
    species_path = species_entry.get()
    output_path = output_entry.get()
    # 调用计算召回率的函数
    process_files_recall(barcode_path, species_path, output_path)
    success_label.config(text="Recall rate processing completed!")

# 特异率处理函数(2-4)
def start_process_specificity():
    barcode_path = barcode_entry_spec.get()
    species_path = species_entry_spec.get()
    output_path = output_entry_spec.get()
    # 调用计算特异率的函数（需要实现）
    process_files_specificity(barcode_path, species_path, output_path)
    success_label_spec.config(text="Specificity processing completed!")

# 一维条形码片段处理函数(2-5)
def start_process_Barcodemaking():
    input_file = barcode_entry_Barcodemaking.get()
    output_file = output_entry_Barcodemaking.get()
    # 调用一维条形码片段处理的函数（需要实现）
    replace_bases_and_format(input_file, output_file)
    success_label_Barcodemaking.config(text="1D code visualization completed!")

# 引物设计处理函数(2-6)
def start_process_Primermaking():
    input_file_path = barcode_entry_Primermaking.get()
    output_excel_path = output_entry_Primermaking.get()
    # 调用引物设计的函数（需要实现）
    process_fasta_for_primers(input_file_path, output_excel_path)
    success_label_Primermaking.config(text="Primer design processing completed!")

# 创建主窗口并设置样式(3-1)
app = tk.Tk()
app.title("Barcoding Processor")
app.geometry("800x400")  # 增加窗口大小
app.configure(bg='#e1d8b2')  # 设置背景颜色

# 创建菜单栏(3-2)
menu_bar = tk.Menu(app)
app.config(menu=menu_bar)

# 设置样式(3-3)
style = ttk.Style()
style.theme_use('clam')  # 使用'clam'主题
style.configure('TLabel', font=('Helvetica', 10), background='#f0f0f0', foreground='#333333')
style.configure('TButton', font=('Helvetica', 10), borderwidth='10')
style.configure('TEntry', font=('Helvetica', 10), borderwidth='10')
style.map('TButton', background=[('active', '#c0c0c0')],foreground=[('active', '#000000'), ('disabled', '#a3a3a3')], borderwidth='10')  # 按钮在激活时的背景色
style.map('TEntry', fieldbackground=[('!disabled', '#ffffff'), ('disabled', '#d9d9d9')])

# 创建简并碱基相关的界面元素(4-1)
frame_DeleteDBs = ttk.Frame(app, padding="20 20 20 20")
frame_DeleteDBs.pack(fill=tk.BOTH, expand=True)

# 简并碱基相关输入和按钮(4-1-1)
ttk.Label(frame_DeleteDBs, text="Input file path(fasta/fas/txt):").grid(column=0, row=0, pady=5)
sequences_entry_DeleteDBs = ttk.Entry(frame_DeleteDBs, width=50)
sequences_entry_DeleteDBs.grid(column=1, row=0, pady=5)
ttk.Button(frame_DeleteDBs, text="Browse...", command=lambda: browse_file(sequences_entry_DeleteDBs)).grid(column=2, row=0, pady=5)

ttk.Label(frame_DeleteDBs, text="Output file path(fasta/fas/txt):").grid(column=0, row=1, pady=5)
output_entry_DeleteDBs = ttk.Entry(frame_DeleteDBs, width=50)
output_entry_DeleteDBs.grid(column=1, row=1, pady=5)
ttk.Button(frame_DeleteDBs, text="Browse...", command=lambda: browse_file(output_entry_DeleteDBs)).grid(column=2, row=1, pady=5)

success_label_DeleteDBs = ttk.Label(frame_DeleteDBs, text="")
success_label_DeleteDBs.grid(column=0, row=2, columnspan=3, pady=5)

# 创建Specificity&Recall rate计算预处理的界面元素(4-2)
frame_SandPpreprocessing = ttk.Frame(app, padding="20 20 20 20")
frame_SandPpreprocessing.pack(fill=tk.BOTH, expand=True)

# Specificity&Recall rate计算预处理相关输入和按钮(4-2-1)
ttk.Label(frame_SandPpreprocessing, text="Sequence + barcode segment input file path(fasta/fas/txt):").grid(column=0, row=0, pady=5)
sequences_entry_SandPpreprocessing = ttk.Entry(frame_SandPpreprocessing, width=50)
sequences_entry_SandPpreprocessing.grid(column=1, row=0, pady=5)
ttk.Button(frame_SandPpreprocessing, text="Browse...", command=lambda: browse_file(sequences_entry_SandPpreprocessing)).grid(column=2, row=0, pady=5)

ttk.Label(frame_SandPpreprocessing, text="Output file path:").grid(column=0, row=1, pady=5)
output_entry_SandPpreprocessing = ttk.Entry(frame_SandPpreprocessing, width=50)
output_entry_SandPpreprocessing.grid(column=1, row=1, pady=5)
ttk.Button(frame_SandPpreprocessing, text="Browse...", command=lambda: browse_folder(output_entry_SandPpreprocessing)).grid(column=2, row=1, pady=5)

success_label_SandPpreprocessing = ttk.Label(frame_SandPpreprocessing, text="")
success_label_SandPpreprocessing.grid(column=0, row=2, columnspan=3, pady=5)

# 创建召回率相关的界面元素(4-3)
frame_recall = ttk.Frame(app, padding="20 20 20 20")
frame_recall.pack(fill=tk.BOTH, expand=True)

# 召回率相关输入和按钮(4-3-1)
ttk.Label(frame_recall, text="Barcode segment input file path(Recall rate):").grid(column=0, row=0, pady=5)
barcode_entry = ttk.Entry(frame_recall, width=50)
barcode_entry.grid(column=1, row=0, pady=5)
ttk.Button(frame_recall, text="Browse...", command=lambda: browse_file(barcode_entry)).grid(column=2, row=0, pady=5)

ttk.Label(frame_recall, text="Species sequence input folder path(Recall rate):").grid(column=0, row=1, pady=5)
species_entry = ttk.Entry(frame_recall, width=50)
species_entry.grid(column=1, row=1, pady=5)
ttk.Button(frame_recall, text="Browse...", command=lambda: browse_folder(species_entry)).grid(column=2, row=1, pady=5)

ttk.Label(frame_recall, text="Output file path(Recall rate):").grid(column=0, row=2, pady=5)
output_entry = ttk.Entry(frame_recall, width=50)
output_entry.grid(column=1, row=2, pady=5)
ttk.Button(frame_recall, text="Browse...", command=lambda: browse_file(output_entry)).grid(column=2, row=2, pady=5)

success_label = ttk.Label(frame_recall, text="")
success_label.grid(column=0, row=3, columnspan=3, pady=5)

# 创建特异率相关的界面元素(4-4)
frame_specificity = ttk.Frame(app, padding="20 20 20 20")
frame_specificity.pack(fill=tk.BOTH, expand=True)

# 特异率相关输入和按钮(4-4-1)
ttk.Label(frame_specificity, text="Barcode segment input file path(Specificity):").grid(column=0, row=0, pady=5)
barcode_entry_spec = ttk.Entry(frame_specificity, width=50)
barcode_entry_spec.grid(column=1, row=0, pady=5)
ttk.Button(frame_specificity, text="Browse...", command=lambda: browse_file(barcode_entry_spec)).grid(column=2, row=0, pady=5)

ttk.Label(frame_specificity, text="Species sequence input folder path(Specificity):").grid(column=0, row=1, pady=5)
species_entry_spec = ttk.Entry(frame_specificity, width=50)
species_entry_spec.grid(column=1, row=1, pady=5)
ttk.Button(frame_specificity, text="Browse...", command=lambda: browse_folder(species_entry_spec)).grid(column=2, row=1, pady=5)

ttk.Label(frame_specificity, text="Output file path(Specificity):").grid(column=0, row=2, pady=5)
output_entry_spec = ttk.Entry(frame_specificity, width=50)
output_entry_spec.grid(column=1, row=2, pady=5)
ttk.Button(frame_specificity, text="Browse...", command=lambda: browse_file(output_entry_spec)).grid(column=2, row=2, pady=5)

success_label_spec = ttk.Label(frame_specificity, text="")
success_label_spec.grid(column=0, row=3, columnspan=3, pady=5)

# 创建Barcodemaking相关的界面元素(4-5)
frame_Barcodemaking = ttk.Frame(app, padding="20 20 20 20")
frame_Barcodemaking.pack(fill=tk.BOTH, expand=True)

# Barcodemaking相关输入和按钮(4-5-1)
ttk.Label(frame_Barcodemaking, text="Barcode nucleotide segment input file path(doc/docx/txt format):").grid(column=0, row=0, pady=5)
barcode_entry_Barcodemaking = ttk.Entry(frame_Barcodemaking, width=50)
barcode_entry_Barcodemaking.grid(column=1, row=0, pady=5)
ttk.Button(frame_Barcodemaking, text="Browse...", command=lambda: browse_file(barcode_entry_Barcodemaking)).grid(column=2, row=0, pady=5)

ttk.Label(frame_Barcodemaking, text="Barcode nucleotide segment output file path(docx format):").grid(column=0, row=1, pady=5)
output_entry_Barcodemaking = ttk.Entry(frame_Barcodemaking, width=50)
output_entry_Barcodemaking.grid(column=1, row=1, pady=5)
ttk.Button(frame_Barcodemaking, text="Browse...", command=lambda: browse_file(output_entry_Barcodemaking)).grid(column=2, row=1, pady=5)

success_label_Barcodemaking = ttk.Label(frame_Barcodemaking, text="")
success_label_Barcodemaking.grid(column=0, row=2, columnspan=3, pady=5)

# 创建Primermaking相关的界面元素(4-6)
frame_Primermaking = ttk.Frame(app, padding="20 20 20 20")
frame_Primermaking.pack(fill=tk.BOTH, expand=True)

# Primermaking相关输入和按钮(4-6-1)
ttk.Label(frame_Primermaking, text="Barcode nucleotide segment input file path(txt format):").grid(column=0, row=0, pady=5)
barcode_entry_Primermaking = ttk.Entry(frame_Primermaking, width=50)
barcode_entry_Primermaking.grid(column=1, row=0, pady=5)
ttk.Button(frame_Primermaking, text="Browse...", command=lambda: browse_file(barcode_entry_Primermaking)).grid(column=2, row=0,pady=5)

ttk.Label(frame_Primermaking, text="Primer output file path(xlsx format):").grid(column=0, row=4, pady=5)
output_entry_Primermaking = ttk.Entry(frame_Primermaking, width=50)
output_entry_Primermaking.grid(column=1, row=4, pady=5)
ttk.Button(frame_Primermaking, text="Browse...", command=lambda: browse_file(output_entry_Primermaking)).grid(column=2, row=4, pady=5)

success_label_Primermaking = ttk.Label(frame_Primermaking, text="")
success_label_Primermaking.grid(column=0, row=2, columnspan=3, pady=5)

# 切换显示的函数(5-1)
def show_DeleteDBs_options():
    frame_DeleteDBs.pack(fill=tk.BOTH, expand=True)
    frame_SandPpreprocessing.pack_forget()
    frame_recall.pack_forget()
    frame_specificity.pack_forget()
    frame_Barcodemaking.pack_forget()
    frame_Primermaking.pack_forget()

def show_SandPpreprocessing_options():
    frame_SandPpreprocessing.pack(fill=tk.BOTH, expand=True)
    frame_DeleteDBs.pack_forget()
    frame_recall.pack_forget()
    frame_specificity.pack_forget()
    frame_Barcodemaking.pack_forget()
    frame_Primermaking.pack_forget()

def show_recall_options():
    frame_recall.pack(fill=tk.BOTH, expand=True)
    frame_DeleteDBs.pack_forget()
    frame_SandPpreprocessing.pack_forget()
    frame_specificity.pack_forget()
    frame_Barcodemaking.pack_forget()
    frame_Primermaking.pack_forget()

def show_specificity_options():
    frame_specificity.pack(fill=tk.BOTH, expand=True)
    frame_DeleteDBs.pack_forget()
    frame_SandPpreprocessing.pack_forget()
    frame_recall.pack_forget()
    frame_Barcodemaking.pack_forget()
    frame_Primermaking.pack_forget()

def show_barcodemaking_options():
    frame_Barcodemaking.pack(fill=tk.BOTH, expand=True)
    frame_DeleteDBs.pack_forget()
    frame_SandPpreprocessing.pack_forget()
    frame_recall.pack_forget()
    frame_specificity.pack_forget()
    frame_Primermaking.pack_forget()

def show_Primermaking_options():
    frame_Primermaking.pack(fill=tk.BOTH, expand=True)
    frame_DeleteDBs.pack_forget()
    frame_SandPpreprocessing.pack_forget()
    frame_recall.pack_forget()
    frame_specificity.pack_forget()
    frame_Barcodemaking.pack_forget()

# 菜单项(5-2)
DeleteDBs_menu = tk.Menu(menu_bar, tearoff=0)
menu_bar.add_cascade(label="DeleteDBs", menu=DeleteDBs_menu)
DeleteDBs_menu.add_command(label="Show Options", command=show_DeleteDBs_options)
DeleteDBs_menu.add_command(label="Start Process", command=start_process_DeleteDBs)

SandPpreprocessing_menu = tk.Menu(menu_bar, tearoff=0)
menu_bar.add_cascade(label="SandPpreprocessing", menu=SandPpreprocessing_menu)
SandPpreprocessing_menu.add_command(label="Show Options", command=show_SandPpreprocessing_options)
SandPpreprocessing_menu.add_command(label="Start Process", command=start_process_SandPpreprocessing)

recall_menu = tk.Menu(menu_bar, tearoff=0)
menu_bar.add_cascade(label="Recall rate", menu=recall_menu)
recall_menu.add_command(label="Show Options", command=show_recall_options)
recall_menu.add_command(label="Start Process", command=start_process_recall)

specificity_menu = tk.Menu(menu_bar, tearoff=0)
menu_bar.add_cascade(label="Specificity", menu=specificity_menu)
specificity_menu.add_command(label="Show Options", command=show_specificity_options)
specificity_menu.add_command(label="Start Process", command=start_process_specificity)

barcodemaking_menu = tk.Menu(menu_bar, tearoff=0)
menu_bar.add_cascade(label="Barcodemaking", menu=barcodemaking_menu)
barcodemaking_menu.add_command(label="Show Options", command=show_barcodemaking_options)
barcodemaking_menu.add_command(label="Start Process", command=start_process_Barcodemaking)

Primermaking_menu = tk.Menu(menu_bar, tearoff=0)
menu_bar.add_cascade(label="Primermaking", menu=Primermaking_menu)
Primermaking_menu.add_command(label="Show Options", command=show_Primermaking_options)
Primermaking_menu.add_command(label="Start Process", command=start_process_Primermaking)

# 默认显示召回率选项(6)
show_DeleteDBs_options()

#以下开始导入功能代码模块部分
#Delete DBs(简并碱基删除)(7-1)
def process_fasta_file(input_file, output_file):
    with open(input_file, 'r') as fi, open(output_file, 'w') as fo:
        for line in fi:
            if '>' in line:
                # 跳过包含 '>' 的行
                fo.write(line)
            else:
                # 清洗序列，并写入输出文件
                cleaned_sequence = re.sub(r'[RYMKSWHBVDNZ]', '', line)
                fo.write(cleaned_sequence)

#Specificity&Recall rate计算预处理(7-2)
def process_and_export_sequences(input_file_path, output_folder_path):
    def parse_fasta(lines):
        sequences, current_seq_id = {}, ""
        for line in lines:
            if line.startswith(">"):
                current_seq_id = line.strip()
                sequences[current_seq_id] = ""
            else:
                sequences[current_seq_id] += line.strip()
        return sequences

    def modify_and_write_sequences(test_sequences, barcode_sequences, output_folder_path):
        # 检查输出目录是否存在，如果不存在则创建
        if not os.path.exists(output_folder_path):
            os.makedirs(output_folder_path)

        output_files = []
        for i, (_, barcode_seq) in enumerate(barcode_sequences):
            gap_indices = {index for index, char in enumerate(barcode_seq) if char == "-"}
            output_filename = f"modified_sequences_{i + 1}.txt"
            full_output_path = os.path.join(output_folder_path, output_filename)
            with open(full_output_path, "w") as output_file:
                for _, test_seq in test_sequences:
                    modified_seq = ''.join(char for index, char in enumerate(test_seq) if index not in gap_indices)
                    output_file.write(modified_seq + "\n")
            output_files.append(full_output_path)
        return output_files

    file_extension = input_file_path.split('.')[-1].lower()
    is_fasta_format = file_extension in ['fasta', 'fas', 'txt']

    with open(input_file_path, 'r') as file:
        if is_fasta_format:
            sequences = parse_fasta(file.readlines())
        else:
            raise ValueError("Unsupported file format")

    root = tk.Tk()
    root.withdraw()  # 隐藏主窗口
    split_index = simpledialog.askinteger("输入", "请输入测试序列和条形码序列的分割索引：", parent=root, minvalue=1,
                                          maxvalue=len(sequences))

    test_sequences = list(sequences.items())[:split_index]
    barcode_sequences = list(sequences.items())[split_index:]

    return modify_and_write_sequences(test_sequences, barcode_sequences, output_folder_path)

#Recall rate(召回率计算)(7-3)
def read_sequences_from_file(file_path):
    """从文件中读取序列，忽略带有 '>' 的行"""
    with open(file_path, 'r') as file:
        return [line.strip() for line in file.readlines() if line.strip() and '>' not in line]

def calculate_recall_rates(barcode_sequence, species_sequences):
    """计算不同阈值下的召回率"""
    total_correct_nucleotides = 0
    barcode_length = len(barcode_sequence)
    num_species = len(species_sequences)
    recall_rates = {threshold: 0 for threshold in range(90, 100)}

    for species_seq in species_sequences:
        # 检查物种核酸序列长度
        if len(species_seq) < barcode_length:
            continue  # 如果物种核酸序列短于条形码片段，则跳过

        correct_nucleotides = sum(1 for i in range(barcode_length) if species_seq[i] == barcode_sequence[i])
        total_correct_nucleotides += correct_nucleotides

        # 计算个体召回率
        individual_recall = correct_nucleotides / barcode_length * 100
        for threshold in recall_rates:
            recall_rates[threshold] += 1 if individual_recall > threshold else 0

    average_nucleotide_recall_rate = (total_correct_nucleotides / barcode_length / num_species) * 100
    average_recall_rates = {threshold: (recall / num_species) * 100 for threshold, recall in recall_rates.items()}

    return round(average_nucleotide_recall_rate, 4), average_recall_rates

def process_files_recall(barcode_file_path, species_folder_path, output_excel_path):
    """处理所有文件并导出结果到Excel文件"""
    barcodes = read_sequences_from_file(barcode_file_path)
    results = []

    for i, barcode in enumerate(barcodes):
        species_file_name = f"modified_sequences_{i + 1}.txt"
        species_file_path = os.path.join(species_folder_path, species_file_name)

        # 检查文件是否存在
        if not os.path.exists(species_file_path):
            continue  # 如果文件不存在，则跳过

        species_sequences = read_sequences_from_file(species_file_path)
        avg_nuc_recall, avg_recall_rates = calculate_recall_rates(barcode, species_sequences)

        result = {
            'Barcode': barcode,
            'Species File': species_file_name,
            'Average Nucleotide Recall Rate': avg_nuc_recall
        }
        result.update({f'Avg Recall Rate > {threshold}%': rate for threshold, rate in avg_recall_rates.items()})
        results.append(result)

    df = pd.DataFrame(results)
    df.to_excel(output_excel_path, index=False)

#Specificity(特异率计算)(7-4)
def read_sequences_from_file(file_path):
    """从文件中读取序列，忽略带有 '>' 的行"""
    with open(file_path, 'r') as file:
        return [line.strip().replace('-', 'N') for line in file.readlines() if line.strip() and '>' not in line]

def calculate_specificity(barcode_sequence, species_sequences):
    total_nucleotide_diffs = 0
    total_species_specificity = 0
    barcode_length = len(barcode_sequence)
    num_species = len(species_sequences)
    specificity_cases = {n_diffs: 0 for n_diffs in range(1, 11)}

    for species_seq in species_sequences:
        # 检查物种核酸序列长度
        if len(species_seq) < barcode_length:
            continue  # 如果物种核酸序列短于条形码片段，则跳过

        nucleotide_diffs = sum(1 for i in range(barcode_length) if species_seq[i] != barcode_sequence[i])
        total_nucleotide_diffs += nucleotide_diffs

        # 计算不同差异数下的特异度
        for n_diffs in specificity_cases:
            specificity_cases[n_diffs] += 100 if nucleotide_diffs >= n_diffs else 0

    average_nucleotide_specificity = (total_nucleotide_diffs / barcode_length / num_species) * 100
    average_specificity_cases = {n_diffs: (specificity / num_species) for n_diffs, specificity in specificity_cases.items()}

    return round(average_nucleotide_specificity, 4), average_specificity_cases

def process_files_specificity(barcode_file_path, species_folder_path, output_excel_path):
    """处理所有文件并导出结果到Excel文件"""
    barcodes = read_sequences_from_file(barcode_file_path)
    results = []

    for i, barcode in enumerate(barcodes):
        species_file_name = f"modified_sequences_{i+1}.txt"
        species_file_path = os.path.join(species_folder_path, species_file_name)

        # 检查文件是否存在
        if not os.path.exists(species_file_path):
            continue  # 如果文件不存在，则跳过

        species_sequences = read_sequences_from_file(species_file_path)
        avg_nuc_spec, avg_specificity_cases = calculate_specificity(barcode, species_sequences)

        result = {
            'Barcode': barcode,
            'Species File': species_file_name,
            'Average Nucleotide Specificity': avg_nuc_spec
        }
        result.update({f'Avg Specificity (Diff >= {n_diffs})': spec for n_diffs, spec in avg_specificity_cases.items()})
        results.append(result)

    df = pd.DataFrame(results)
    df.to_excel(output_excel_path, index=False)

#Barcodemaking(1D条形码片段可视化)(7-5)
def replace_bases_and_format(input_file, output_file):
    # 确定后缀以选择处理方式
    file_suffix = input_file.split('.')[-1].lower()
    text = ""

    # 读取文件内容
    if file_suffix in ['doc', 'docx']:
        doc = Document(input_file)
        text = '\n'.join([p.text for p in doc.paragraphs])
    elif file_suffix == 'txt':
        with open(input_file, 'r') as file:
            text = file.read()
    else:
        raise ValueError("Unsupported file format")

    # 替换碱基并记录替换位置和碱基类型
    base_replacements = {
        'A': ('|', 'Showcard Gothic', 14, RGBColor(0x80, 0x76, 0xA3)),  # 四号字体大约对应12磅
        'T': ('|', 'Showcard Gothic', 14, RGBColor(0xFF, 0x00, 0x00)),
        'G': ('|', 'Showcard Gothic', 16, RGBColor(0x56, 0x98, 0xC3)),  # 三号字体大约对应16磅
        'C': ('|', 'Showcard Gothic', 16, RGBColor(0x00, 0xB0, 0x50)),
    }

    # 创建新的Word文档
    doc = Document()
    # 设置整体样式
    style = doc.styles['Normal']
    font = style.font
    font.name = 'Showcard Gothic'

    # 将文本添加到文档
    for line in text.split('\n'):
        p = doc.add_paragraph()
        p.alignment = WD_PARAGRAPH_ALIGNMENT.LEFT
        for char in line:
            run = p.add_run(char)
            if char in base_replacements:
                replacement, font_name, font_size, color = base_replacements[char]
                run.text = replacement
                run.font.name = font_name
                run.font.size = Pt(font_size)
                run.font.color.rgb = color

    # 保存新的Word文档
    doc.save(output_file)

#Primermaking(条形码片段引物设计)(7-6)
def process_fasta_for_primers(input_file_path, output_excel_path, primer_length=18):
    """
    Processes a FASTA file to design primers and calculate their properties,
    then saves the data to an Excel file.
    """
    def parse_fasta(file_path):
        sequences = {}
        seq_name = ""
        seq = ""
        with open(file_path, 'r') as file:
            for line in file:
                if line.startswith('>'):
                    if seq_name:
                        sequences[seq_name] = seq
                    seq_name = line.strip()
                    seq = ""
                else:
                    seq += line.strip()
            if seq_name:
                sequences[seq_name] = seq
        return sequences

    def reverse_complement(seq):
        complement = {'A': 'T', 'T': 'A', 'G': 'C', 'C': 'G'}
        return ''.join(complement[base] for base in reversed(seq))

    def calculate_gc_content(primer):
        gc_count = primer.count('G') + primer.count('C')
        return (gc_count / len(primer)) * 100

    def calculate_tm(primer):
        g = primer.count('G')
        c = primer.count('C')
        a = primer.count('A')
        t = primer.count('T')
        return 4 * (g + c) + 2 * (a + t)

    sequences = parse_fasta(input_file_path)
    primer_data = []

    for name, seq in sequences.items():
        forward_primer = seq[:primer_length]
        reverse_primer = reverse_complement(seq[-primer_length:])
        primer_data.append({
            'Region': name,
            'Forward Primer': forward_primer,
            'Reverse Primer': reverse_primer,
            'Forward GC%': calculate_gc_content(forward_primer),
            'Reverse GC%': calculate_gc_content(reverse_primer),
            'Forward Tm': calculate_tm(forward_primer),
            'Reverse Tm': calculate_tm(reverse_primer)
        })

    primer_df = pd.DataFrame(primer_data)
    primer_df.to_excel(output_excel_path, index=False)

#该代码必须要在末尾，相当于可视化界面出栈函数(8)
app.mainloop()